package java.net;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Enumeration;
import java.io.IOException;

public class MulticastSocket extends DatagramSocket
{
    private boolean interfaceSet;
    private Object ttlLock;
    private Object infLock;
    private InetAddress infAddress;
    private static final NetworkInterface defNetIntf;
    
    public MulticastSocket() throws IOException {
        this(new InetSocketAddress(0));
    }
    
    public MulticastSocket(int n) throws IOException {
        this(new InetSocketAddress(n));
    }
    
    public MulticastSocket(SocketAddress socketAddress) throws IOException {
        super((SocketAddress)null);
        this.ttlLock = new Object();
        this.infLock = new Object();
        this.infAddress = null;
        this.setReuseAddress(true);
        if (socketAddress != null) {
            try {
                this.bind(socketAddress);
            }
            finally {
                if (!this.isBound()) {
                    this.close();
                }
            }
        }
    }
    
    @Deprecated
    public void setTTL(byte ttl) throws IOException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        this.getImpl().setTTL(ttl);
        return;
    }
    
    public void setTimeToLive(int timeToLive) throws IOException {
        if (timeToLive < 0 || timeToLive > 255) {
            throw new IllegalArgumentException("ttl out of range");
        }
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        this.getImpl().setTimeToLive(timeToLive);
        return;
    }
    
    @Deprecated
    public byte getTTL() throws IOException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        return this.getImpl().getTTL();
    }
    
    public int getTimeToLive() throws IOException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        return this.getImpl().getTimeToLive();
    }
    
    public void joinGroup(InetAddress maddr) throws IOException {
        synchronized (this.infLock) {
            if (!this.interfaceSet && MulticastSocket.defNetIntf != null) {
                if (maddr == null) {
                    throw new NullPointerException("Multicast address is null");
                }
                this.joinGroup(new InetSocketAddress(maddr, 0), MulticastSocket.defNetIntf);
                return;
            }
        }
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        this.checkAddress(maddr, "joinGroup");
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkMulticast(maddr);
        }
        if (!maddr.isMulticastAddress()) {
            throw new SocketException("Not a multicast address");
        }
        NetworkInterface default1 = NetworkInterface.getDefault();
        if (!this.interfaceSet && default1 != null) {
            this.setNetworkInterface(default1);
        }
        this.getImpl().join(maddr);
        return;
    }
    
    public void leaveGroup(InetAddress maddr) throws IOException {
        synchronized (this.infLock) {
            if (!this.interfaceSet && MulticastSocket.defNetIntf != null) {
                if (maddr == null) {
                    throw new NullPointerException("Multicast address is null");
                }
                this.leaveGroup(new InetSocketAddress(maddr, 0), MulticastSocket.defNetIntf);
                return;
            }
        }
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        this.checkAddress(maddr, "leaveGroup");
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkMulticast(maddr);
        }
        if (!maddr.isMulticastAddress()) {
            throw new SocketException("Not a multicast address");
        }
        this.getImpl().leave(maddr);
        return;
    }
    
    public void joinGroup(SocketAddress socketAddress, NetworkInterface networkInterface) throws IOException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        if (socketAddress == null || !(socketAddress instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Unsupported address type");
        }
        if (this.oldImpl) {
            throw new UnsupportedOperationException();
        }
        this.checkAddress(((InetSocketAddress)socketAddress).getAddress(), "joinGroup");
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkMulticast(((InetSocketAddress)socketAddress).getAddress());
        }
        if (!((InetSocketAddress)socketAddress).getAddress().isMulticastAddress()) {
            throw new SocketException("Not a multicast address");
        }
        this.getImpl().joinGroup(socketAddress, networkInterface);
        return;
    }
    
    public void leaveGroup(SocketAddress socketAddress, NetworkInterface networkInterface) throws IOException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        if (socketAddress == null || !(socketAddress instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Unsupported address type");
        }
        if (this.oldImpl) {
            throw new UnsupportedOperationException();
        }
        this.checkAddress(((InetSocketAddress)socketAddress).getAddress(), "leaveGroup");
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkMulticast(((InetSocketAddress)socketAddress).getAddress());
        }
        if (!((InetSocketAddress)socketAddress).getAddress().isMulticastAddress()) {
            throw new SocketException("Not a multicast address");
        }
        this.getImpl().leaveGroup(socketAddress, networkInterface);
        return;
    }
    
    public void setInterface(InetAddress infAddress) throws SocketException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        this.checkAddress(infAddress, "setInterface");
        synchronized (this.infLock) {
            this.getImpl().setOption(16, infAddress);
            this.infAddress = infAddress;
            this.interfaceSet = true;
        }
        return;
    }
    
    public InetAddress getInterface() throws SocketException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        synchronized (this.infLock) {
            InetAddress inetAddress = (InetAddress)this.getImpl().getOption(16);
            if (this.infAddress == null) {
                return inetAddress;
            }
            if (inetAddress.equals(this.infAddress)) {
                return inetAddress;
            }
            try {
                Enumeration<InetAddress> inetAddresses = NetworkInterface.getByInetAddress(inetAddress).getInetAddresses();
                while (inetAddresses.hasMoreElements()) {
                    if (((InetAddress)inetAddresses.nextElement()).equals(this.infAddress)) {
                        return this.infAddress;
                    }
                    continue;
                }
                this.infAddress = null;
                return inetAddress;
            }
            catch (Exception ex) {
                return inetAddress;
            }
        }
    }
    
    public void setNetworkInterface(NetworkInterface networkInterface) throws SocketException {
        synchronized (this.infLock) {
            this.getImpl().setOption(31, networkInterface);
            this.infAddress = null;
            this.interfaceSet = true;
        }
    }
    
    public NetworkInterface getNetworkInterface() throws SocketException {
        NetworkInterface networkInterface = (NetworkInterface)this.getImpl().getOption(31);
        if (networkInterface.getIndex() == 0 || networkInterface.getIndex() == -1) {
            InetAddress[] array = { InetAddress.anyLocalAddress() };
            return new NetworkInterface(array[0].getHostName(), 0, array);
        }
        return networkInterface;
    }
    
    public void setLoopbackMode(boolean b) throws SocketException {
        this.getImpl().setOption(18, Boolean.valueOf(b));
    }
    
    public boolean getLoopbackMode() throws SocketException {
        return (boolean)this.getImpl().getOption(18);
    }
    
    @Deprecated
    public void send(DatagramPacket datagramPacket, byte b) throws IOException {
        if (this.isClosed()) {
            throw new SocketException("Socket is closed");
        }
        this.checkAddress(datagramPacket.getAddress(), "send");
        synchronized (this.ttlLock) {
            synchronized (datagramPacket) {
                if (this.connectState == 0) {
                    SecurityManager securityManager = System.getSecurityManager();
                    if (securityManager != null) {
                        if (datagramPacket.getAddress().isMulticastAddress()) {
                            securityManager.checkMulticast(datagramPacket.getAddress(), b);
                        }
                        else {
                            securityManager.checkConnect(datagramPacket.getAddress().getHostAddress(), datagramPacket.getPort());
                        }
                    }
                }
                else {
                    InetAddress address = datagramPacket.getAddress();
                    if (address == null) {
                        datagramPacket.setAddress(this.connectedAddress);
                        datagramPacket.setPort(this.connectedPort);
                    }
                    else if (!address.equals(this.connectedAddress) || datagramPacket.getPort() != this.connectedPort) {
                        throw new SecurityException("connected address and packet address differ");
                    }
                }
                int ttl = this.getTTL();
                try {
                    if (b != ttl) {
                        this.getImpl().setTTL(b);
                    }
                    this.getImpl().send(datagramPacket);
                }
                finally {
                    if (b != ttl) {
                        this.getImpl().setTTL((byte)ttl);
                    }
                }
            }
        }
        return;
    }
    
    static {
        String str = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return System.getProperty("jdk.net.defaultMulticastInterface");
            }
        });
        NetworkInterface byName = null;
        if (str != null) {
            try {
                byName = NetworkInterface.getByName(str);
                if (byName == null) {
                    System.err.println("WARNING: cannot find network interface " + str);
                }
                else {
                    System.err.println("INFO: network interface set to " + str);
                }
            }
            catch (SocketException ex) {
                System.err.println("ERROR: failed to find network interface " + str);
            }
        }
        defNetIntf = byName;
    }
}
